#include <iostream>
#include <fstream>
#include <cmath>
#include <sstream> // Bibliotheque pour les flux vers des chaines de caracteres.




//// Les parametres du probleme ////

const double c = 0.1; // Coefficient de diffusion
const double T_fin = 1; // On calcule la solution jusqu'a T_fin
const int nombre_images = 10; // Nombre de fichiers de sortie

const int M = 2001; // Nombre de pas de temps
const double dt = T_fin/M; // Pas de temps
const int N = 100; // Nombre de pas d'espace. La solution au temps t va etre representee par un tableau de taille *N+1*
const double dx = 1./N; // Pas d'espace




// Condition initiale
double u0(double x)
{
  return cos(3*M_PI*x); 
}


// Calcule une iteration du schema a partir de u1,
// et stocke le resultat dans u2
void iteration(double const *u1, double *u2)
{
  u2[0] = 1;
  u2[N] = -1;
  
  for (int k=1; k<N; k++)
	u2[k] = u1[k] + c*dt/(dx*dx) * (u1[k+1] + u1[k-1] - 2*u1[k]);
}


// Affiche le vecteur u dans un flux
void affiche(std::ostream &flux, double *u)
{  
  for (int k=0; k<=N; k++)
	flux << k*dx << " " << u[k]<< "\n";
}


int main(void)
{
  // Si les pas de temps/espace choisis rendent le schema instable
  // on le signale, et on ne fait pas le calcul
  if (dx*dx < 2*c*dt)
	{
	  std::cout << "Schema instable. Abandon.\nPour un pas d'espace de "<< dx << ", le pas de temps doit etre inferieur a "<< (dx*dx)/(2*c) << ", soit " << (int)T_fin*2*c/(dx*dx) << " pas jusqu'au temps "<< T_fin <<".\n";
	  return 0;
	}

  
  double *u = new double[N+1];
  double *u_1 = new double[N+1];
  
  std::ofstream sortie;
  std::stringstream nom_fichier;
  // nom_fichier est un flux vers une chaine de caracteres. Il va permettre de donner des noms differents a chaque fichier de sortie.

  for (int k=0; k<=N; k++)
	// On initialise
	{
	  u[k] = u0(k*dx);
	}
  
  for (int n=0; n<=M; n++)
	{
	  
	  /*
		On envoie la solution a certains instants
		dans un fichier, "fichier_xxx.dat".
	  */
	  if (n % (M/(nombre_images-1)) == 0)
		{

		  // Les lignes suivantes permettent d'ouvrir en ecriture un fichier "fichier_xxx.dat"
		  // ou "xxx" est remplace par un nombre different a chaque fois.
		  
		  nom_fichier.str("");
		  nom_fichier << "fichier_" << 10 + n/(M/(nombre_images-1)) << ".dat";
		  sortie.open(nom_fichier.str().data());

		  
		  sortie << "# Temps t = " << n*dt << "\n";
		  affiche(sortie, u);
		  sortie.close();
		}

	  
	  // Une iteration du schema (la valeur a l'instant t+dt est stockee dans u_1)...
	  iteration(u,u_1);

	  // ... et on recopie u_1 dans u.
	  for (int k=0; k<=N; k++)
		{
		  u[k] = u_1[k];
		}
	}

  delete []u;
  delete []u_1;
  
  return 0;
}
