#include "tableau.hpp"



/* Exercice 2 : chercher le maximum d'un tableau */

double maximum(double *tableau, int taille)
{
  double M = tableau[0];
  /* Il ne faut pas initialiser a une valeur arbitraire (par exemple 0),
	 car la fonction ne pourrait alors pas trouver le maximum d'un
	 tableau dont toutes les valeurs sont inferieures a cette valeur arbitraire.
  */

  for (int i=1; i<taille; i++)
	if (tableau[i]>M)
	  M = tableau[i];
  
  return M;  
}



/* Exercice 3 : remplir le tableau avec des variables aleatoires */

void remplit_tableau(double *tableau, int taille, std::mt19937 &gen)
{
  std::uniform_real_distribution<double>U(0,1);
  for (int i=0; i<taille; i++)
	tableau[i] = U(gen);
}

void histogramme(double *tableau, int taille)
{
  double histo[10]; // On peut utiliser un tableau statique car la taille est connue.
  
  for (int i=0; i<10; i++)
	histo[i] = 0;

  for (int i=0; i<taille; i++)
	{
	  int k = 10*tableau[i]; //Comme k est un int, il prend automatiquement comme valeur la partie entière de 10*tableau[i].
	  if (0<=k and k<10)
		histo[k] ++;
	}
  
  for (int i=0; i<10; i++)
	{
	  std::cout << "[ " << i << "/10, " << i+1 << "/10 ] : " << histo[i] << '\n';
	}
  
}





/* Exercice 4 : trier un tableau */

/*
  Le tri par insertion : on parcourt le tableau, en inserant
  chaque nouvel element a sa place parmi les elements deja vus
  (penser a la maniere dont on trie un jeu de cartes).

  Cet algorithme est assez naturel, mais a une complexite en n^2,
  ou n est la taille du tableau.
*/
void tri_insertion(double *tableau, int taille)
{
  double tampon;
  for (int i=1 ; i<taille ; i++)
	{
	  int j = i-1;
	  // A chaque passage de la boucle, les i premiers elements sont tries
	  while (tableau[j]>tableau[j+1])
		{
		  // On "fait descendre" le ieme element jusqu'a sa place
		  tampon = tableau[j];
		  tableau[j] = tableau[j+1];
		  tableau[j+1] = tampon;
		  j--;
		}
	}
  
}



/*
  Le tri rapide : on choisit un element du tableau, appele "pivot"
  (ici le premier, mais il peut etre plus efficace de le choisir au
  hasard). On place au debut du tableau tous les elements inferieurs
  au pivot et a la fin tous les elements superieurs. Le pivot est donc
  a la bonne place. On trie ensuite de maniere recursive les deux
  sous-tableaux situes de part et d'autre du pivot.

  Cet algorithme est plus efficace que le precedent : sa complexite est
  en nlog(n).
*/
void tri_rapide(double *tableau, int taille)
{
  tri_rapide_aux(tableau, 0, taille-1);
}
// La fonction auxilliaire tri_rapide_aux trie les elements du tableau entre les positions "debut" et "fin".
void tri_rapide_aux(double *tableau, int debut, int fin)
{
  if (fin <= debut)
	return; // Si le tableau ne fait que 0 ou 1 case, il est deja trie.

  
  int position_pivot = debut;
  double pivot = tableau[debut];
  /* On place tous les elements inferieurs a pivot a gauche, et tous les
	 elements superieurs a droite. Par consequent, pivot se trouve a la
	 bonne place.
  */
  for (int i=debut+1; i<=fin; i++)
	if (tableau[i]<pivot)
	  {
		tableau[position_pivot] = tableau[i];
		tableau[i] = tableau[position_pivot+1];
		tableau[position_pivot+1] = pivot;
		position_pivot++;
	  }

  tri_rapide_aux(tableau, debut, position_pivot-1);
  tri_rapide_aux(tableau, position_pivot+1, fin);
}
