#include "vecteur.hpp"



/* Exercice 2 : chercher le maximum d'un vecteur */

double maximum(std::vector<double> const &vect)
{
  double M = vect[0];
  /* Il ne faut pas initialiser a une valeur arbitraire (par exemple 0),
	 car la fonction ne pourrait alors pas trouver le maximum d'un
	 vect dont toutes les valeurs sont inferieures a cette valeur arbitraire.
  */

  for (unsigned i=1; i<vect.size(); i++)
	if (vect[i]>M)
	  M = vect[i];
  
  return M;  
}



/* Exercice 3 : remplir le vecteur avec des variables aleatoires */

void remplit_vecteur(std::vector<double> &vect, std::mt19937 &gen)
{
  std::uniform_real_distribution<double>U(0,1);
  for (unsigned i=0; i<vect.size(); i++)
	vect[i] = U(gen);
}

void histogramme(std::vector<double> const &vect)
{
  std::vector<double> histo(10); // On peut utiliser un vecteur statique car la taille est connue.
  
  for (int i=0; i<10; i++)
	histo[i] = 0;

  for (unsigned i=0; i<vect.size(); i++)
	{
	  int k = 10*vect[i]; //Comme k est un int, il prend automatiquement comme valeur la partie entière de 10*vect[i].
	  if (0<=k and k<10)
		histo[k] ++;
	}
  
  for (int i=0; i<10; i++)
	{
	  std::cout << "[ " << i << "/10, " << i+1 << "/10 ] : " << histo[i] << '\n';
	}
  
}





/* Exercice 4 : trier un vecteur */

/*
  Le tri par insertion : on parcourt le vecteur, en inserant
  chaque nouvel element a sa place parmi les elements deja vus
  (penser a la maniere dont on trie un jeu de cartes).

  Cet algorithme est assez naturel, mais a une complexite en n^2,
  ou n est la taille du vecteur.
*/
void tri_insertion(std::vector<double> &vect)
{
  double tampon;
  for (unsigned i=1 ; i<vect.size() ; i++)
	{
	  int j = i-1;
	  // A chaque passage de la boucle, les i premiers elements sont tries
	  while (vect[j]>vect[j+1])
		{
		  // On "fait descendre" le ieme element jusqu'a sa place
		  tampon = vect[j];
		  vect[j] = vect[j+1];
		  vect[j+1] = tampon;
		  j--;
		}
	}
  
}



/*
  Le tri rapide : on choisit un element du vecteur, appele "pivot"
  (ici le premier, mais il peut etre plus efficace de le choisir au
  hasard). On place au debut du vecteur tous les elements inferieurs
  au pivot et a la fin tous les elements superieurs. Le pivot est donc
  a la bonne place. On trie ensuite de maniere recursive les deux
  sous-vecteurx situes de part et d'autre du pivot.

  Cet algorithme est plus efficace que le precedent : sa complexite est
  en nlog(n).
*/
void tri_rapide(std::vector<double> &vect)
{
  tri_rapide_aux(vect, 0, vect.size()-1);
}
// La fonction auxilliaire tri_rapide_aux trie les elements du vecteur entre les positions "debut" et "fin".
void tri_rapide_aux(std::vector<double> &vect, int debut, int fin)
{
  if (fin <= debut)
	return; // Si le vecteur ne fait que 0 ou 1 case, il est deja trie.

  
  int position_pivot = debut;
  double pivot = vect[debut];
  /* On place tous les elements inferieurs a pivot a gauche, et tous les
	 elements superieurs a droite. Par consequent, pivot se trouve a la
	 bonne place.
  */
  for (int i=debut+1; i<=fin; i++)
	if (vect[i]<pivot)
	  {
		vect[position_pivot] = vect[i];
		vect[i] = vect[position_pivot+1];
		vect[position_pivot+1] = pivot;
		position_pivot++;
	  }

  tri_rapide_aux(vect, debut, position_pivot-1);
  tri_rapide_aux(vect, position_pivot+1, fin);
}
